package com.ruoyi.project.hit.util.webSocket;

import com.alibaba.fastjson.JSONObject;


import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.time.Instant;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@ServerEndpoint("/echo/{id}")
@Component
public class WsServerEndpoint {
    public static Map<String,WsServerEndpoint> wsServerEndpointMap = new ConcurrentHashMap<>();

    private Session session;

    private String id;
    // 收到消息
    @OnMessage
    public void onMessage(String message) throws Exception {
        JSONObject jsonObject = JSONObject.parseObject(message);
        String id= (String) jsonObject.get("id");
        String msg=(String) jsonObject.get("msg");

        if (message.equalsIgnoreCase("bye")) {
            // 由服务器主动关闭连接。状态码为 NORMAL_CLOSURE（正常关闭）。
            this.session.close(new CloseReason(CloseReason.CloseCodes.NORMAL_CLOSURE, "Bye"));;
            return;
        }
        if(id!=null)
        {
            if("-1".equals(id))
            {
                sendAllUser("1");
            }
            else {
                sendToUser(id, msg);
            }
        }

//        this.session.getAsyncRemote().sendText("["+ Instant.now().toEpochMilli() +"] Hello " + message);
    }

    // 连接打开
    @OnOpen
    public void onOpen(@PathParam(value = "id") String id, Session session, EndpointConfig endpointConfig){
        System.out.println("开启连接");
        // 保存 session 到对象
        this.session = session;
        this.id = id;
        wsServerEndpointMap.put(id,this);
        System.out.println(wsServerEndpointMap.toString());
    }

    // 连接关闭
    @OnClose
    public void onClose(CloseReason closeReason){
        wsServerEndpointMap.remove(this);
        System.out.println("断开连接");
    }

    // 连接异常
    @OnError
    public void onError(Throwable throwable) throws IOException {
        System.out.println(throwable);
        // 关闭连接。状态码为 UNEXPECTED_CONDITION（意料之外的异常）
        this.session.close(new CloseReason(CloseReason.CloseCodes.UNEXPECTED_CONDITION, throwable.getMessage()));
    }

    public void sendMessage(String message) throws Exception {
        this.session.getAsyncRemote().sendText(message);
    }
    public void sendToUser(String id, String message) throws Exception {
        if(wsServerEndpointMap.containsKey(id)){
            wsServerEndpointMap.get(id).session.getAsyncRemote().sendText(message);
        }
    }
    public void sendAllUser(String message) throws Exception{
        for (String s : wsServerEndpointMap.keySet()) {
            wsServerEndpointMap.get(s).sendMessage(message);
        }
    }


}
